#include <math.h>
#include <stdlib.h>
#include "nrutil.c"


float **y, *xx; 

float K, L, G;


void rk4 (float y[], 
		  float dydx[], 
		  int n, 
		  float x, 
		  float h, 
		  float yout[],
		  void (*derivs) (float, 
		                  float [], 
						  float [])
		  )
/* 
Given values for the variables y[1..n] and their derivatives dydx[1..n] known at x, use the
fourth-order Runge-Kutta method to advance the solution over an interval h and return the
incremented variables as yout[1..n], which need not be a distinct array from y. The user
supplies the routine derivs(x,y,dydx), which returns derivatives dydx at x. 
*/
{
	int i;
	float xh,hh,h6,*dym,*dyt,*yt;
	
	dym=vector(1,n);
	dyt=vector(1,n);
	yt=vector(1,n);

	hh=h*0.5;
	h6=h/6.0;
	xh=x+hh;
	
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dydx[i]; // First step.
	
	(*derivs)(xh,yt,dyt);                     // Second step.
	for (i=1;i<=n;i++) yt[i]=y[i]+hh*dyt[i];

	(*derivs)(xh,yt,dym);                     // Third step.
	for (i=1;i<=n;i++) {
		yt[i]=y[i]+h*dym[i];
		dym[i] += dyt[i];
	}

	(*derivs)(x+h,yt,dyt);                    // Fourth step.
	for (i=1;i<=n;i++)                        // Accumulate increments with proper weights. 
		yout[i]=y[i]+h6*(dydx[i]+dyt[i]+2.0*dym[i]);

	free_vector(yt,1,n);
	free_vector(dyt,1,n);
	free_vector(dym,1,n);
}



void rkdumb (float vstart[], 
			 int nvar, 
			 float x1, 
			 float x2, 
			 int nstep,
			 void (*derivs)(float, 
							float [], 
							float [])
			 )
/*
Starting from initial values vstart[1..nvar] known at x1 use fourth-order Runge-Kutta
to advance nstep equal increments to x2. The user-supplied routine derivs(x,v,dvdx)
evaluates derivatives. Results are stored in the global variables y[1..nvar][1..nstep+1]
and xx[1..nstep+1].
*/
{
	void rk4 (float y[], float dydx[], int n, float x, float h, float yout[],
			  void (*derivs)(float, float [], float []));
	int i,k;
	float x,h;
	float *v,*vout,*dv;

	v=vector(1,nvar);
	vout=vector(1,nvar);
	dv=vector(1,nvar);

	for (i=1;i<=nvar;i++) { // Load starting values.
		v[i]=vstart[i];
		y[i][1]=v[i];
	}

	xx[1]=x1;
	x=x1;
	h=(x2-x1)/nstep;
	
	for (k=1;k<=nstep;k++) { // Take nstep steps.
	
		(*derivs)(x,v,dv);
		rk4(v,dv,nvar,x,h,vout,derivs);
		
		if ((float)(x+h) == x) nrerror("Step size too small in routine rkdumb");
		
		x += h;
		xx[k+1]=x;			// Store intermediate steps.
		
		for (i=1;i<=nvar;i++) {
			v[i]=vout[i];
			y[i][k+1]=v[i];
		}
	}
	
	free_vector(dv,1,nvar);
	free_vector(vout,1,nvar);
	free_vector(v,1,nvar);
}



void euler_step (int N,
				 float x, 
	 			 float y[], 
				 float h, 
				 void (*f) (float, float [], float []), 
				 float ynext[])
{
	float *dyperdx;
	int n;

	dyperdx = vector (1, N);

	(*f) (x, y, dyperdx);

	for (n=1; n<=N; n++)	
	{
		ynext[n] = y[n] + h*dyperdx[n];  
	}

	free_vector (dyperdx, 1, N);
}


void euler_drv (float x0, 
				float x1, 
				int xstep,
				int N,
				float y0[],
				void (*f) (float, float [], float []))
{
	float h, *yn, *yn1;
	int i, n;

	h = (x1-x0)/xstep;

	yn = vector (1, N);
	yn1 = vector (1, N);

	xx[1] = x0;

	for (n=1; n<=N; n++) 
	{	
		y[n][1] = y0[n];
		yn[n] = y0[n];
	}

	for (i=1; i<=xstep; i++)
	{
		euler_step (N,
					xx[i], 
					yn, 
					h, 
					f, 
					yn1);

		xx[i+1] = xx[i]+h;
		
		for (n=1; n<=N; n++) 
		{	
			y[n][i+1] = yn1[n];
			yn[n] = yn1[n];
		}
	}

	free_vector (yn, 1, N);
	free_vector (yn1, 1, N);

}


void f_rugos_inga (float x, float y[], float dyperdx[])
{
	float s;

	s = sqrt( y[1]*y[1] + y[2]*y[2] );
										// y[1]: x(t), y[2]: y(t), y[3]: vx(t), y[4]: vy(t)
	dyperdx[1] = y[3];
	dyperdx[2] = y[4];
	if (s>0)
	{	
		dyperdx[3] = (K*(L-s)*y[1])/s;
		dyperdx[4] = (K*(L-s)*y[2])/s - G;
	}
	else nrerror("Zeroval osztas: f_rugos_inga !");
}


void main (int argc, char *argv[])
{

	int i, xstep, n, N, method;
	float x0, x1, *y0, nrg;


	G = 10.0;					// a fizikai problema alapertelmezett parameterei
	K = 10.0;
	L = 1.0;

	x0 = 0.0;
	x1 = 1.0;
	xstep = 1024;

	N = 4;
	y0 = vector (1, N);			// kezdeti ertekek es alapertelmezett ertekei
	y0[1] = L;
	y0[2] = 0;
	y0[3] = 0;
	y0[4] = 0;

	for (i=1; i<argc; i++ ){
		
		switch(i){
	    case 1: 
			x0 = atof(argv[1]); 
			break;
		case 2:
			x1 = atof(argv[2]); 
			break;
		case 3:
			xstep = atoi(argv[3]); 
			break;
		case 4:
			y0[1] = atof(argv[4]);
			break;
		case 5:
			y0[2] = atof(argv[5]);
			break;
		case 6:
			y0[3] = atof(argv[6]);
			break;
		case 7:
			y0[4] = atof(argv[7]);
			break;
		case 8:
			K = atof(argv[8]);
			break;
		case 9:
			L = atof(argv[9]);
			break;
		case 10:
			method = atoi(argv[10]);
			break;
		} 
	}

	xx = vector (1, xstep+1);
	y = matrix (1, N, 1, xstep+1);


	switch(method)
	{
	 case 1: 
		euler_drv (	x0, 
					x1, 
					xstep,
					N,
					y0,
					f_rugos_inga);
		break;
	 
	 case 2:
		rkdumb (y0, 
				N, 
				x0, 
				x1, 
				xstep,
				f_rugos_inga);
		break;
	}


	for (i=1;i<=xstep+1;i++)
	{
		nrg = (sqrt(y[1][i]*y[1][i] + y[2][i]*y[2][i]) - L);
		nrg = (y[3][i]*y[3][i] + y[4][i]*y[4][i])/2 
			+ G*y[2][i] 
			+ K*(nrg*nrg)/2;
		
		printf("%f", xx[i]);
		
		for (n=1; n<=N; n++) 
		{	
			printf("\t%f", y[n][i]);
		}

		printf("\t%f\n", nrg);
	
	}

	free_vector (xx, 1, xstep+1);
	free_vector (y0, 1, xstep+1);
	free_matrix (y, 1, N, 1, xstep+1);
}
